@@ -213,18 +213,23 @@ void initconsnode(node **p, node **grbg, node *q, long len, long nodei,
     break;
   case treewt:
     if (!eoln(intree)) {
-      fscanf(intree, "%lf", &trweight);
-      getch(ch, parens, intree);
-      if (*ch != ']') {
-        printf("\n\nERROR: Missing right square bracket\n\n");
-        exxit(-1);
-      } else {
+      if (fscanf(intree, "%lf", &trweight) == 1) {
         getch(ch, parens, intree);
-        if (*ch != ';') {
-          printf("\n\nERROR: Missing semicolon after square brackets\n\n");
+        if (*ch != ']') {
+          printf("\n\nERROR: Missing right square bracket\n\n");
           exxit(-1);
+        } else {
+          getch(ch, parens, intree);
+          if (*ch != ';') {
+            printf("\n\nERROR: Missing semicolon after square brackets\n\n");
+            exxit(-1);
+          }
         }
       }
+      else {
+        printf("\n\nERROR: Expecting tree weight in last comment field\n\n");
+        exxit(-1);
+      }
     }
     break;
   case unittrwt:
@@ -1170,6 +1175,7 @@ void missingname(node *p){
   namesCheckTable();
 } /* missingname */
 
+
 void gdispose(node *p)
 {
   /* go through tree throwing away nodes */
@@ -1214,9 +1220,6 @@ void reroot(node *outgroup, long *nextnode)
   node *p, *q;
   double newv;
 
-  
-  
-  
   /* count root's children & find last */
   p = root;
   i = 0;
@@ -1369,9 +1372,8 @@ void reordertips()
   for (i = 0; i < spp-1; i++) {
     for (j = i + 1; j < spp; j++) {
       if (samename(nayme[i], nodep[j]->nayme)) {
-        /* Don't exchange the nodes themselves, just switch the pointers in
-         * nodep[] and set index accordingly for each node. -IDR
-         */
+        /* switch the pointers in
+         * nodep[] and set index accordingly for each node. */
         t = nodep[i];
         
         nodep[i] = nodep[j];
