@@ -276,6 +276,7 @@ void getoptions()
 #ifdef WIN32
     phyFillScreenColor();
 #endif
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     if (ch == '\n')
@@ -378,6 +379,7 @@ void getoptions()
           loopcount2 = 0;
           do {
             printf(" (type D or W)\n");
+            fflush(stdout);
             scanf("%c%*[^\n]", &ch2);
             getchar();
             if (ch2 == '\n')
@@ -439,6 +441,7 @@ void getoptions()
 #ifdef WIN32
       phyFillScreenColor();
 #endif
+      fflush(stdout);
       scanf("%lf%*[^\n]", &cv);
       getchar();
       countup(&loopcount, 10);
@@ -467,6 +470,7 @@ void getoptions()
         loopcount = 0;
         do {
           printf("Fraction of invariant sites?\n");
+          fflush(stdout);
           scanf("%lf%*[^\n]", &invarfrac);
           getchar();
           countup (&loopcount, 10);
@@ -2065,6 +2069,7 @@ void summarize()
     for (i = 1; i <= nmlngth + 3; i++)
       putc(' ', outfile);
     for (i = 0; i < sites; i++) {
+      mm = 0;
       sum = 0.0;
       for (j = 0; j < rcategs; j++)
         if (marginal[i][j] > sum) {
@@ -2236,7 +2241,7 @@ void dnaml_unroot(node* root, node** nodep, long nonodes)
     }
     free(r->x);
     r->x = NULL;
-    chucktreenode(&grbg, r);
+    chuck(&grbg, r);
     curtree.nodep[spp] = q;
   } else { /* Bifurcating root - remove entire root fork */
     /* Join oldlen on each side of root */
@@ -2280,9 +2285,9 @@ void dnaml_unroot(node* root, node** nodep, long nonodes)
     free(root->next->x);
     free(root->next->next->x);
 
-    chucktreenode(&grbg,root->next->next);
-    chucktreenode(&grbg,root->next);
-    chucktreenode(&grbg,root);
+    chuck(&grbg,root->next->next);
+    chuck(&grbg,root->next);
+    chuck(&grbg,root);
   }
 } /* dnaml_unroot */
 
@@ -2298,7 +2303,8 @@ void maketree()
   inittable();
 
   if (usertree) {
-    openfile(&intree,INTREE,"input tree file", "r",progname,intreename);
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree file", "rb",progname,intreename);
     inittable_for_usertree (intree);
     numtrees = countsemic(&intree);
     if(numtrees > MAXSHIMOTREES)
@@ -2573,8 +2579,11 @@ int main(int argc, Char *argv[])
     getinput();
     if (ith == 1)
       firstset = false;
-    for (jumb = 1; jumb <= njumble; jumb++)
+    if (usertree)
       maketree();
+    else
+      for (jumb = 1; jumb <= njumble; jumb++)
+        maketree();
   }
 
   clean_up();
