@@ -175,6 +175,7 @@ void getoptions()
 #ifdef WIN32
     phyFillScreenColor();
 #endif
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     if (ch == '\n')
@@ -223,6 +224,7 @@ void getoptions()
 #ifdef WIN32
             phyFillScreenColor();
 #endif
+            fflush(stdout);
             scanf("%c%*[^\n]", &ch2);
             getchar();
             if (ch2 == '\n')
@@ -256,6 +258,7 @@ void getoptions()
 #ifdef WIN32
             phyFillScreenColor();
 #endif
+            fflush(stdout);
             scanf("%c%*[^\n]", &ch2);
             getchar();
             if (ch2 == '\n')
@@ -274,6 +277,7 @@ void getoptions()
 #ifdef WIN32
           phyFillScreenColor();
 #endif
+          fflush(stdout);
           scanf("%ld%*[^\n]", &maxtrees);
           if (maxtrees > MAXNUMTREES)
             maxtrees = MAXNUMTREES;        
@@ -1438,7 +1442,8 @@ void maketree()
       }
     }
   } else {
-    openfile(&intree,INTREE,"input tree", "r",progname,intreename);
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree", "rb",progname,intreename);
     numtrees = countsemic(&intree);
     if (numtrees > 2)
       initseed(&inseed, &inseed0, seed);
