@@ -69,6 +69,7 @@ boolean lastrearr;
 double nsteps[maxuser];
 node *there;
 long fullset;
+long shimotrees;
 bitptr zeroanc, oneanc;
 long *place;
 Char ch;
@@ -159,6 +160,7 @@ void getoptions()
 #ifdef WIN32
     phyFillScreenColor();
 #endif
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     uppercase(&ch);
@@ -207,6 +209,7 @@ void getoptions()
 #ifdef WIN32
             phyFillScreenColor();
 #endif
+            fflush(stdout);
             scanf("%c%*[^\n]", &ch2);
             getchar();
             if (ch2 == '\n')
@@ -862,13 +865,15 @@ void maketree()
       }
     }
   } else {
-    openfile(&intree,INTREE,"input tree file", "r",progname,intreename);
-    fscanf(intree, "%ld", &numtrees);
-    getc(intree);
-    if (numtrees > 2) {
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree file", "rb",progname,intreename);
+
+    numtrees = countsemic(&intree);
+    if (numtrees > 2){
       initseed(&inseed, &inseed0, seed);
       printf("\n");
     }
+
     if (treeprint) {
       fprintf(outfile, "User-defined tree");
       if (numtrees > 1)
@@ -989,4 +994,3 @@ int main(int argc, Char *argv[])
 #endif
   return 0;
 }  /* Dollo or polymorphism parsimony by uphill search */
-
