@@ -1033,7 +1033,8 @@ void buildtree()
     break;
 
   case use:
-    openfile(&intree,INTREE,"input tree file", "r",progname,intreename);
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree file", "rb",progname,intreename);
     names = (boolean *)Malloc(spp*sizeof(boolean));
     firsttree = true;                       /**/
     nodep = NULL;                           /**/
@@ -1042,9 +1043,8 @@ void buildtree()
     zeros = (long *)Malloc(chars*sizeof(long));         /**/
     for (i = 0; i < chars; i++)             /**/
       zeros[i] = 0;                         /**/
-    treeread(intree, &root, treenode, &goteof, &firsttree,
-                nodep, &nextnode, &haslengths,
-                &grbg, initdolmovenode); /*debug*/
+    treeread(intree, &root, treenode, &goteof, &firsttree, nodep, &nextnode,
+        &haslengths, &grbg, initdolmovenode,false,nonodes);
     for (i = spp; i < (nonodes); i++) {
       p = treenode[i];
       for (j = 1; j <= 3; j++) {
