@@ -188,6 +188,7 @@ void getoptions()
 #ifdef WIN32
     phyFillScreenColor();
 #endif
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     uppercase(&ch);
@@ -1005,7 +1006,8 @@ void maketree()
       setuptipf(which, &curtree);
     if (eoln(infile))
       scan_eoln(infile);
-    openfile(&intree,INTREE,"input tree file","r",progname,intreename);
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree file","rb",progname,intreename);
     numtrees = countsemic(&intree);
     if (numtrees > MAXNUMTREES) {
       printf("\nERROR: number of input trees is read incorrectly from %s\n",
