@@ -168,6 +168,7 @@ void getoptions()
 #ifdef WIN32
     phyFillScreenColor();
 #endif
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     if (ch == '\n')
@@ -212,6 +213,7 @@ void getoptions()
 #ifdef WIN32
             phyFillScreenColor();
 #endif
+            fflush(stdout);
             scanf("%c%*[^\n]", &ch2);
             getchar();
             if (ch2 == '\n')
@@ -245,6 +247,7 @@ void getoptions()
 #ifdef WIN32
             phyFillScreenColor();
 #endif
+            fflush(stdout);
             scanf("%c%*[^\n]", &ch2);
             getchar();
             if (ch2 == '\n')
@@ -263,10 +266,11 @@ void getoptions()
 #ifdef WIN32
           phyFillScreenColor();
 #endif
+          fflush(stdout);
           scanf("%ld%*[^\n]", &maxtrees);
+          getchar();
           if (maxtrees  > MAXNUMTREES)
             maxtrees = MAXNUMTREES;
-          getchar();
           countup(&loopcount2, 10);
         } while (maxtrees < 1);
         break;
@@ -1469,7 +1473,8 @@ void maketree()
       }
     }
   } else {
-    openfile(&intree,INTREE,"input tree", "r",progname,intreename);
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree", "rb",progname,intreename);
     numtrees = countsemic(&intree);
     if (numtrees > MAXNUMTREES) {
       printf(
