@@ -356,6 +356,7 @@ void getoptions()
     }
     printf(
    "\nAre these settings correct? (type Y or the letter for one to change)\n");
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     uppercase(&ch);
@@ -400,6 +401,7 @@ void getoptions()
         loopcount2 = 0;
         do {
           printf("type U, M, V, F, or Y\n");
+          fflush(stdout);
           scanf("%c%*[^\n]", &ch);
           getchar();
           if (ch == '\n')
@@ -442,6 +444,7 @@ void getoptions()
 #ifdef WIN32
             phyFillScreenColor();
 #endif
+            fflush(stdout);
             scanf("%c%*[^\n]", &ch2);
             getchar();
             if (ch2 == '\n')
@@ -1846,7 +1849,8 @@ void maketree()
       }
     }
   } else {
-    openfile(&intree,INTREE,"input tree file", "r",progname,intreename);
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree file", "rb",progname,intreename);
     numtrees = countsemic(&intree);
     if (treeprint) {
       fprintf(outfile, "User-defined tree");
