@@ -4,6 +4,8 @@
    Permission is granted to copy and use this program provided no fee is
    charged for it and provided that this copyright notice is not removed. */
 
+#include <float.h>
+
 #include "phylip.h"
 #include "seq.h"
 
@@ -34,7 +36,7 @@ void   copymatrix(transmatrix, transmatrix);
 void   maketrans(double, boolean);
 void   branchtrans(long, double);
 double evaluate(tree *, node *);
-void   nuview(node *);
+boolean nuview(node *);
 void   makenewv(node *);
 void   update(node *);
 void   smooth(node *);
@@ -48,13 +50,21 @@ void   buildsimpletree(tree *);
 void   addtraverse(node *, node *, boolean);
 void   rearrange(node *, node *);
 void   restml_coordinates(node *, double, long *,double *, double *);
+void   restml_fprintree(FILE *fp);
 void   restml_printree(void);
 
 double sigma(node *, double *);
-void   describe(node *);
+void   fdescribe(FILE *, node *);
 void   summarize(void);
 void   restml_treeout(node *);
+static phenotype2 restml_pheno_new(long endsite, long sitelength);
+static void restml_pheno_delete(phenotype2 x2);
+void initrestmlnode(node **p, node **grbg, node *q, long len, long nodei,
+    long *ntips, long *parens, initops whichinit, pointarray treenode,
+    pointarray nodep, Char *str, Char *ch, FILE *intree);
+static void restml_unroot(node* root, node** nodep, long nonodes);
 void   inittravtree(tree* t,node *);
+static void adjust_lengths_r(node *p);
 void   treevaluate(void);
 void   maketree(void);
 void   globrearrange(void); 
@@ -64,6 +74,7 @@ sitelike2 init_sitelike(long sitelength);
 void   free_sitelike(sitelike2 sl);
 void   copy_sitelike(sitelike2 dest, sitelike2 src,long sitelength);
 void   reallocsites(void);
+static void set_branchnum(node *p, long branchnum);
 void   alloctrans(tree *t, long nonodes, long sitelength);
 long   get_trans(tree* t);
 void   free_trans(tree* t, long trans);
@@ -71,6 +82,7 @@ void   free_all_trans(tree* t);
 void   alloclrsaves(void);
 void   freelrsaves(void);
 void   resetlrsaves(void);
+void   cleanup(void);
 /* function prototypes */
 #endif
 
@@ -127,8 +139,44 @@ Char ch;
 /* variables added to keep treeread2() happy */
 boolean goteof;
 double trweight;
-boolean haslengths;
+node *grbg = NULL;
 
+#ifdef DEBUG
+void checknode(node *p) {
+  assert(p != NULL);
+  assert(p->back != NULL);
+  assert(p->back->back == p);
+  //assert(p->v == p->back->v);
+  //assert(p->branchnum > 0);
+  //assert(p->branchnum == p->back->branchnum);
+}
+
+void checktree_r(node *p) {
+  node *q;
+
+  checknode(p);
+  if (!p->tip) {
+    for (q = p->next; q != p; q = q->next) {
+      checktree_r(q->back);
+    }
+  }
+}
+
+void checktree() {
+  checktree_r(curtree.start);
+  checktree_r(curtree.start->back);
+}
+#endif /* DEBUG */
+
+
+static void
+set_branchnum(node *p, long branchnum)
+{
+  assert(p != NULL);
+  assert(branchnum > 0);
+  p->branchnum = branchnum;
+}
+  
 
 void alloctrans(tree *t, long nonodes, long sitelength)
 {
@@ -152,6 +200,7 @@ void alloctrans(tree *t, long nonodes, long sitelength)
 long get_trans(tree* t)
 {
   long ret;
+  assert(t->transindex >= 0);
   ret = t->freetrans[t->transindex];
   t->transindex--;
   return ret;
@@ -289,6 +338,7 @@ void getoptions()
     printf("  4       Write out trees onto tree file?  %s\n",
            (trout ? "Yes" : "No"));
     printf("\n  Y to accept these or type the letter for one to change\n");
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     if (ch == '\n')
@@ -323,6 +373,7 @@ void getoptions()
         loopcount2 = 0;
         do {
           printf("New Sitelength?\n");
+          fflush(stdout);
           scanf("%ld%*[^\n]", &sitelength);
           getchar();
           if (sitelength < 1)
@@ -493,7 +544,7 @@ void doinit()
     tempcurve[i] = (double *)Malloc((sitelength+1) * sizeof(double));
   setuppie();
   alloctrans(&curtree, nonodes2, sitelength);
-  alloctree(&curtree.nodep, nonodes2, false);
+  alloctree(&curtree.nodep, nonodes2, usertree);
   allocrest();
   if (usertree)
     return;
@@ -507,6 +558,27 @@ void doinit()
 }  /* doinit */
 
 
+void cleanup() {
+  long i, j;
+
+  for (i = 0; i < NTEMPMATS; i++) {
+    for (j = 0; j <= sitelength; j++)
+      free(tempmatrix[i][j]);
+    free(tempmatrix[i]);
+  }
+  free(tempmatrix);
+  tempmatrix = NULL;
+  for (i = 0; i <= sitelength; i++) {
+    free(tempslope[i]);
+    free(tempcurve[i]);
+  }
+  free(tempslope);
+  tempslope = NULL;
+  free(tempcurve);
+  tempcurve = NULL;
+}
+
+
 void inputoptions()
 {
   /* read the options information */
@@ -812,14 +884,14 @@ void getinput()
   if ( !firstset ) freelrsaves();
   makeweights();
   alloclrsaves();
-  setuptree2(&curtree);
   if (!usertree) {
+    setuptree2(&curtree);
     setuptree2(&priortree);
     setuptree2(&bestree);
     if (njumble > 1) 
       setuptree2(&bestree2);
   }
-  allocx2(nonodes2, endsite+1, sitelength, curtree.nodep, false);
+  allocx2(nonodes2, endsite+1, sitelength, curtree.nodep, usertree);
   if (!usertree) {
     allocx2(nonodes2, endsite+1, sitelength, priortree.nodep, 0);
     allocx2(nonodes2, endsite+1, sitelength, bestree.nodep, 0);
@@ -925,8 +997,10 @@ double evaluate(tree *tr, node *p)
   x2 = init_sitelike(sitelength);
   sum = 0.0;
   q = p->back;
-  if ( p->initialized  == false && p->tip == false)  nuview(p);
-  if ( q->initialized  == false && q->tip == false)  nuview(q);
+
+  nuview(p);
+  nuview(q);
+
   y = p->v;
   branchnum = p->branchnum;
   copy_sitelike(x1,p->x2[0],sitelength);
@@ -976,63 +1050,50 @@ double evaluate(tree *tr, node *p)
 }  /* evaluate */
 
 
-void nuview(node *p)
+boolean nuview(node *p)
 {
   /* recompute fractional likelihoods for one part of tree */
   long i, j, k, lowlim;
-  double sumq, sumr;
-  node *q, *r;
+  double sumq;
+  node *q, *s;
   sitelike2 xq, xr, xp;
-  transmatrix tempq, tempr;
-  double *tq, *tr;
+  double **tempq = NULL;
+  double *tq     = NULL;
 
+  if (p->tip)
+    return false;
   xq = init_sitelike(sitelength);
   xr = init_sitelike(sitelength);
   xp = init_sitelike(sitelength);
-  if (!p->next->back->tip && !p->next->back->initialized)
-    nuview (p->next->back);
-  if (!p->next->next->back->tip && !p->next->next->back->initialized)
-    nuview (p->next->next->back);
-  tempq = (transmatrix)Malloc((sitelength+1) * sizeof(double *));
-  tempr = (transmatrix)Malloc((sitelength+1) * sizeof(double *));
-  for (i=0;i<=sitelength;++i){
-    tempq[i] = (double *)Malloc((sitelength+1) * sizeof (double));
-    tempr[i] = (double *)Malloc((sitelength+1) * sizeof (double));
+  for (s = p->next; s != p; s = s->next) {
+    if ( nuview(s->back) )
+      p->initialized = false;
   }
-  if (trunc8)
-    lowlim = 0;
-  else
-    lowlim = 1;
-  q = p->next->back;
-  r = p->next->next->back;
-  copymatrix(tempq,curtree.trans[q->branchnum - 1]);
-  copymatrix(tempr,curtree.trans[r->branchnum - 1]);
+
+  if (p->initialized)
+    return false;
+
+  lowlim = trunc8 ? 0 : 1;
+
   for (i = lowlim; i <= endsite; i++) {
-    copy_sitelike(xq,q->x2[i],sitelength);
-    copy_sitelike(xr,r->x2[i],sitelength);
-    for (j = 0; j <= sitelength; j++) {
-      sumq = 0.0;
-      sumr = 0.0;
-      tq = tempq[j];
-      tr = tempr[j];
-      for (k = 0; k <= sitelength; k++) {
-        sumq += tq[k] * xq[k];
-        sumr += tr[k] * xr[k];
+    xp = p->x2[i];
+    for (j = 0; j <= sitelength; j++)
+      xp[j] = 1.0;
+    for (s = p->next; s != p; s = s->next) {
+      q = s->back;
+      tempq = curtree.trans[q->branchnum - 1];
+      xq = q->x2[i];
+      for (j = 0; j <= sitelength; j++) {
+        sumq = 0.0;
+        tq = tempq[j];
+        for (k = 0; k <= sitelength; k++)
+          sumq += tq[k] * xq[k];
+        xp[j] *= sumq;
       }
-      xp[j] = sumq * sumr;
     }
-    copy_sitelike(p->x2[i],xp,sitelength);
   }
-  for (i=0;i<=sitelength;++i){
-    free(tempq[i]);
-    free(tempr[i]);
-  }
-  free(tempq);
-  free(tempr);
-  p->initialized = true;
-  free_sitelike(xq);
-  free_sitelike(xr);
-  free_sitelike(xp);
+
+  return true;
 }  /* nuview */
 
 
@@ -1047,8 +1108,6 @@ void makenewv(node *p)
   sitelike2 xx1, xx2;
   double *tm, *ts, *tc;
 
-  xx1 = init_sitelike(sitelength);
-  xx2 = init_sitelike(sitelength);
   q = p->back;
   y = p->v;
   yorig = y;
@@ -1067,8 +1126,8 @@ void makenewv(node *p)
     curve = 0.0;
     maketrans(y, nr);
     for (i = lowlim; i <= endsite; i++) {
-      copy_sitelike(xx1,p->x2[i],sitelength);
-      copy_sitelike(xx2,q->x2[i],sitelength);
+      xx1 = p->x2[i];
+      xx2 = q->x2[i];
       sum = 0.0;
       sums = 0.0;
       sumc = 0.0;
@@ -1134,19 +1193,15 @@ void makenewv(node *p)
   q->v = yold;
   branchtrans(p->branchnum, yold);
   curtree.likelihood = oldlike;
-  free_sitelike(xx1);
-  free_sitelike(xx2);
-
 }  /* makenewv */
 
 
 void update(node *p)
 {
   /* improve branch length and views for one branch */
-  if (!p->tip && !p->initialized)
-    nuview(p);
-  if (!p->back->tip && !p->back->initialized)
-    nuview(p->back);
+  nuview(p);
+  nuview(p->back);
+
   if ( !(usertree && lengths) ) {
     makenewv(p);
     if (smoothit ) {
@@ -1200,9 +1255,9 @@ void insert_(node *p, node *q)
   r->v = q->v;
   r->back->v = r->v;
  
-  q->back->branchnum = q->branchnum;
-  r->branchnum = get_trans(&curtree);
-  r->back->branchnum = r->branchnum;
+  set_branchnum(q->back, q->branchnum);
+  set_branchnum(r, get_trans(&curtree));
+  set_branchnum(r->back, r->branchnum);
   branchtrans(q->branchnum, q->v);
   branchtrans(r->branchnum, r->v);
 
@@ -1234,7 +1289,7 @@ void restml_re_move(node **p, node **q)
   *q = (*p)->next->back;
   hookup(*q, (*p)->next->next->back);
   free_trans(&curtree,(*q)->back->branchnum);
-  (*q)->back->branchnum = (*q)->branchnum;
+  set_branchnum((*q)->back, (*q)->branchnum);
   (*q)->v = 0.75*(1 - (1 - 1.333333*(*q)->v) * (1 - 1.333333*(*p)->next->v));
 
   if ( (*q)->v > 1 - epsilon)
@@ -1268,7 +1323,7 @@ void restml_copynode(node *c, node *d)
   /* copy a node */
   long i;
 
-  d->branchnum = c->branchnum;
+  set_branchnum(d, c->branchnum);
   
   for ( i = 0 ; i <= endsite ; i++)
     copy_sitelike(d->x2[i],c->x2[i],sitelength);
@@ -1349,8 +1404,8 @@ void buildnewtip(long m,tree *tr)
   hookup(tr->nodep[m - 1], p);
   p->v = initialv;
   p->back->v = initialv;
-  p->branchnum = get_trans(tr);
-  p->back->branchnum = p->branchnum;
+  set_branchnum(p, get_trans(tr));
+  set_branchnum(p->back, p->branchnum);
   branchtrans(p->branchnum, initialv);
 }  /* buildnewtip */
 
@@ -1365,8 +1420,8 @@ void buildsimpletree(tree *tr)
   tr->nodep[enterorder[1] - 1]->v = initialv;
   branchtrans(enterorder[1], initialv);
   branch = get_trans(tr);
-  tr->nodep[enterorder[0] - 1]->branchnum = branch;
-  tr->nodep[enterorder[1] - 1]->branchnum = branch;
+  set_branchnum(tr->nodep[enterorder[0] - 1], branch);
+  set_branchnum(tr->nodep[enterorder[1] - 1], branch);
   buildnewtip(enterorder[2], tr);
   insert_(tr->nodep[enterorder[2] - 1]->back, tr->nodep[enterorder[1] - 1]);
   tr->start = tr->nodep[enterorder[2]-1]->back;
@@ -1403,9 +1458,14 @@ void addtraverse(node *p, node *q, boolean contin)
     q->v = vsave;
     q->back->v = vsave;
     free_trans(&curtree,q->back->branchnum);
-    q->back->branchnum = q->branchnum;
+    set_branchnum(q->back, q->branchnum);
     copymatrix (curtree.trans[q->branchnum - 1], tempmatrix[1]);
-    curtree.likelihood = bestyet;
+#ifdef DEBUG
+    evaluate(&curtree, curtree.start);
+    assert( close(bestyet, curtree.likelihood) );
+#endif
+    /* curtree.likelihood = bestyet; */
+    evaluate(&curtree, curtree.start);
   }
   if (!q->tip && contin) {
     /* assumes bifurcation (OK) */
@@ -1413,7 +1473,7 @@ void addtraverse(node *p, node *q, boolean contin)
     addtraverse(p, q->next->next->back, contin);
   }
   if ( contin && q == curtree.root ) {
-    /* FIXME!! curtree.root->back == NULL? */
+    /* FIXME!! curtree.root->back == NULL?  curtree.root == NULL? */
     addtraverse(p,q->back,contin);
   }
 }  /* addtraverse */
@@ -1508,7 +1568,7 @@ void printnode(node* p)
 {
   if (p->back)
   printf("p->index = %3ld, p->back->index = %3ld, p->branchnum = %3ld,evaluates"
-      " to %lf\n",p->index,p->back->index,p->branchnum,evaluate(&curtree,p));
+      " to %f\n",p->index,p->back->index,p->branchnum,evaluate(&curtree,p));
   
   else 
   printf("p->index = %3ld, p->back->index =none, p->branchnum = %3ld,evaluates"
@@ -1550,7 +1610,10 @@ void rearrange(node *p, node *pp)
     return;
 
   } else /* if !p->tip && !p->back->tip */ {
+    /*
+    evaluate(&curtree, curtree.start);
     bestyet = curtree.likelihood;
+    */
 
     if (p->back->next != pp)
       r = p->back->next;
@@ -1603,11 +1666,10 @@ void rearrange(node *p, node *pp)
         r->next->next->back->v = r->next->next->v;
         p->next->back->v = p->next->v;
         p->next->next->back->v = p->next->next->v;
-        r->back->branchnum = r->branchnum;
-        r->next->back->branchnum = r->next->branchnum;
-        r->next->next->back->branchnum = r->next->next->branchnum;
-        p->next->back->branchnum = p->next->branchnum;
-        p->next->next->back->branchnum = p->next->next->branchnum;
+        set_branchnum(r->back, r->branchnum);
+        set_branchnum(r->next->back, r->next->branchnum);
+        set_branchnum(p->next->back, p->next->branchnum);
+        set_branchnum(p->next->next->back, p->next->next->branchnum);
         copymatrix (curtree.trans[r->branchnum-1], tempmatrix[2]);
         copymatrix (curtree.trans[r->next->branchnum-1], tempmatrix[3]);
         copymatrix (curtree.trans[r->next->next->branchnum-1], tempmatrix[4]);
@@ -1667,24 +1729,31 @@ void restml_coordinates(node *p, double lengthsum, long *tipy,
 }  /* restml_coordinates */
 
 
-void restml_printree()
+void restml_fprintree(FILE *fp)
 {
   /* prints out diagram of the tree */
   long tipy,i;
   double scale, tipmax, x;
 
-  putc('\n', outfile);
+  putc('\n', fp);
   if (!treeprint)
     return;
-  putc('\n', outfile);
+  putc('\n', fp);
   tipy = 1;
   tipmax = 0.0;
   restml_coordinates(curtree.start, 0.0, &tipy,&tipmax,&x);
   scale = 1.0 / (tipmax + 1.000);
   for (i = 1; i <= tipy - down; i++)
-    drawline2(i, scale, curtree);
-  putc('\n', outfile);
-}  /* restml_printree */
+    fdrawline2(fp, i, scale, &curtree);
+  putc('\n', fp);
+}  /* restml_fprintree */
+
+
+void restml_printree()
+{
+  restml_fprintree(outfile);
+}
+  
 
 
 double sigma(node *q, double *sumlr)
@@ -1814,61 +1883,65 @@ double sigma(node *q, double *sumlr)
 }  /* sigma */
 
 
-void describe(node *p)
+void fdescribe(FILE *fp, node *p)
 {
   /* print out information on one branch */
   double sumlr;
   long i;
   node *q;
   double s;
+  double realv;
 
   q = p->back;
-  fprintf(outfile, "%4ld      ", q->index - spp);
-  fprintf(outfile, "    ");
+  fprintf(fp, "%4ld      ", q->index - spp);
+  fprintf(fp, "    ");
   if (p->tip) {
     for (i = 0; i < nmlngth; i++)
-      putc(nayme[p->index - 1][i], outfile);
+      putc(nayme[p->index - 1][i], fp);
   } else
-    fprintf(outfile, "%4ld      ", p->index - spp);
+    fprintf(fp, "%4ld      ", p->index - spp);
   if (q->v >= 0.75)
-    fprintf(outfile, "     infinity");
-  else
-    fprintf(outfile, "%13.5f", -0.75 * log(1 - 1.333333 * q->v));
+    fprintf(fp, "     infinity");
+  else {
+    realv = -0.75 * log(1 - 4.0/3.0 * q->v);
+    fprintf(fp, "%13.5f", realv);
+  }
   if (p->iter) {
     s = sigma(q, &sumlr);
     if (s < 0.0)
-      fprintf(outfile, "     (     zero,    infinity)");
+      fprintf(fp, "     (     zero,    infinity)");
     else {
-      fprintf(outfile, "     (");
+      fprintf(fp, "     (");
       if (q->v - s <= 0.0)
-        fprintf(outfile, "     zero");
+        fprintf(fp, "     zero");
       else
-        fprintf(outfile, "%9.5f", -0.75 * log(1 - 1.333333 * (q->v - s)));
-      putc(',', outfile);
+        fprintf(fp, "%9.5f", -0.75 * log(1 - 1.333333 * (q->v - s)));
+      putc(',', fp);
       if (q->v + s >= 0.75)
-        fprintf(outfile, "    infinity");
+        fprintf(fp, "    infinity");
       else
-        fprintf(outfile, "%12.5f", -0.75 * log(1 - 1.333333 * (q->v + s)));
-      putc(')', outfile);
+        fprintf(fp, "%12.5f", -0.75 * log(1 - 1.333333 * (q->v + s)));
+      putc(')', fp);
       }
     if (sumlr > 1.9205)
-      fprintf(outfile, " *");
+      fprintf(fp, " *");
     if (sumlr > 2.995)
-      putc('*', outfile);
+      putc('*', fp);
     }
   else
-    fprintf(outfile, "            (not varied)");
-  putc('\n', outfile);
+    fprintf(fp, "            (not varied)");
+  putc('\n', fp);
   if (!p->tip) {
-    describe(p->next->back);
-    describe(p->next->next->back);
+    for (q = p->next; q != p; q = q->next)
+      fdescribe(fp, q->back);
   }
-}  /* describe */
+}  /* fdescribe */
 
 
 void summarize()
 {
   /* print out information on branches of tree */
+  node *q;
 
   fprintf(outfile, "\nremember: ");
   if (outgropt)
@@ -1880,9 +1953,9 @@ void summarize()
   fprintf(outfile, "      Approx. Confidence Limits\n");
   fprintf(outfile, " -------        ---            ------");
   fprintf(outfile, "      ------- ---------- ------\n");
-  describe(curtree.start->next->back);
-  describe(curtree.start->next->next->back);
-  describe(curtree.start->back);
+  for (q = curtree.start->next; q != curtree.start; q = q->next)
+    fdescribe(outfile, q->back);
+  fdescribe(outfile, curtree.start->back);
   fprintf(outfile, "\n     *  = significantly positive, P < 0.05\n");
   fprintf(outfile, "     ** = significantly positive, P < 0.01\n\n\n");
 }  /* summarize */
@@ -1952,13 +2025,167 @@ void restml_treeout(node *p)
 }  /* restml_treeout */
 
 
+static phenotype2
+restml_pheno_new(long endsite, long sitelength)
+{
+  phenotype2 ret;
+  long k, l;
+
+  endsite++;
+  ret = (phenotype2)Malloc(endsite*sizeof(sitelike2));
+  for (k = 0; k < endsite; k++) {
+    ret[k] = Malloc((sitelength + 1) * sizeof(double));
+    for (l = 0; l < sitelength; l++)
+      ret[k][l] = 1.0;
+  }
+  
+  return ret;
+}
+
+
+static void
+restml_pheno_delete(phenotype2 x2)
+{
+  long k;
+
+  for (k = 0; k < endsite+1; k++)
+    free(x2[k]);
+  free(x2);
+}
+
+
+void initrestmlnode(node **p, node **grbg, node *q, long len, long nodei,
+                        long *ntips, long *parens, initops whichinit,
+                        pointarray treenode, pointarray nodep, Char *str, 
+                        Char *ch, FILE *intree)
+{
+  /* initializes a node */
+  boolean minusread;
+  double valyew, divisor;
+
+  switch (whichinit) {
+  case bottom:
+    gnu(grbg, p);
+    (*p)->index = nodei;
+    (*p)->tip = false;
+    (*p)->branchnum = 0;
+    (*p)->x2 = restml_pheno_new(endsite, sitelength);
+    nodep[(*p)->index - 1] = (*p);
+    break;
+  case nonbottom:
+    gnu(grbg, p);
+    (*p)->x2 = restml_pheno_new(endsite, sitelength);
+    (*p)->index = nodei;
+    break;
+  case tip:
+    match_names_to_data (str, nodep, p, spp);
+    break;
+  case iter:
+    (*p)->initialized = false;
+    (*p)->v = initialv;
+    (*p)->iter = true;
+    if ((*p)->back != NULL){
+      (*p)->back->iter = true;
+      (*p)->back->v = initialv;  
+      (*p)->back->initialized = false;
+    }
+    break;
+  case length:
+    processlength(&valyew, &divisor, ch, &minusread, intree, parens);
+    (*p)->v = valyew / divisor;
+    (*p)->iter = false;
+    if ((*p)->back != NULL) {
+      (*p)->back->v = (*p)->v;
+      (*p)->back->iter = false;
+    }
+    break;
+  case hsnolength:
+    break;
+  default:        /* cases hslength, treewt, unittrwt */
+    break;
+  }
+} /* initrestmlnode */
+
+
+static void
+restml_unroot(node* root, node** nodep, long nonodes) 
+{
+  node *p,*r,*q;
+  double newl;
+  long i;
+  long numsibs;
+  
+  numsibs = count_sibs(root);
+
+  if ( numsibs > 2 ) {
+    q = root;
+    r = root;
+    while (!(q->next == root))
+      q = q->next;
+    q->next = root->next;
+
+    /* FIXME?
+    for(i=0 ; i < endsite ; i++){
+      free(r->x[i]);
+      r->x[i] = NULL;
+    }
+    free(r->x);
+    r->x = NULL;
+    */
+
+    chuck(&grbg, r);
+    curtree.nodep[spp] = q;
+  } else { /* Bifurcating root - remove entire root fork */
+    /* Join v on each side of root */
+    newl = root->next->v + root->next->next->v;
+    root->next->back->v = newl;
+    root->next->next->back->v = newl;
+
+    /* Connect root's children */
+    hookup(root->next->back, root->next->next->back);
+
+    /* Move nodep entries down one and set indices */
+    for ( i = spp; i < nonodes-1; i++ ) {
+      p = nodep[i+1];
+      nodep[i] = p;
+      nodep[i+1] = NULL;
+      if ( nodep[i] == NULL ) /* This may happen in a
+                                 multifurcating intree */
+        break;
+      do {
+        p->index = i+1;
+        p = p->next;
+      } while (p != nodep[i]);
+    }
+
+    /* Free protx arrays from old root */
+    /*
+    for(i=0 ; i < endsite ; i++){
+      free(root->x[i]);
+      free(root->next->x[i]);
+      free(root->next->next->x[i]);
+      root->x[i] = NULL;
+      root->next->x[i] = NULL;
+      root->next->next->x[i] = NULL;
+    }
+    free(root->x);
+    free(root->next->x);
+    free(root->next->next->x);
+    */
+
+    chuck(&grbg,root->next->next);
+    chuck(&grbg,root->next);
+    chuck(&grbg,root);
+  }
+} /* dnaml_unroot */
+
 void inittravtree(tree* t,node *p)
 { /* traverse tree to set initialized and v to initial values */
   node* q; 
 
   if ( p->branchnum == 0) {
-    p->branchnum = get_trans(t);
-    p->back->branchnum = p->branchnum;
+    set_branchnum(p, get_trans(t));
+    set_branchnum(p->back, p->branchnum);
   }
   p->initialized = false;
   p->back->initialized = false;
@@ -1983,23 +2210,33 @@ double adjusted_v(double v) {
   return 3.0/4.0 * (1.0-exp(-4.0/3.0 * v));
 }
 
-void adjust_lengths(tree* t) {
-  long i;
-  for ( i = 0 ; i < spp ; i++) {
-    t->nodep[i]->v = adjusted_v(t->nodep[i]->v);
-  }
-  for ( i = spp ; i < nonodes2 ; i++) {
-    t->nodep[i]->v = adjusted_v(t->nodep[i]->v);
-    t->nodep[i]->next->v = adjusted_v(t->nodep[i]->next->v);
-    t->nodep[i]->next->next->v = adjusted_v(t->nodep[i]->next->next->v);
+
+static void
+adjust_lengths_r(node *p)
+{
+  node *q; 
+
+  p->v = adjusted_v(p->v);
+  p->back->v = p->v;
+  if (!p->tip) {
+    for (q = p->next; q != p; q = q->next)
+      adjust_lengths_r(q->back);
   }
 }
 
+
+void adjust_lengths(tree *t)
+{
+  assert(t->start->back->tip);
+  adjust_lengths_r(t->start);
+}
+
+
 void treevaluate()
 {
   /* find maximum likelihood branch lengths of user tree */
   long i;
-  unroot(&curtree,nonodes2);
+
   if ( lengths)  
     adjust_lengths(&curtree);
   nonodes2--;
@@ -2021,9 +2258,11 @@ void maketree()
 {
   /* construct and rearrange tree */
   long i,j;
+  long nextnode;
 
   if (usertree) {
-    openfile(&intree,INTREE,"input tree file","r",progname,intreename);
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree,INTREE,"input tree file","rb",progname,intreename);
     numtrees = countsemic(&intree);
     if(numtrees > MAXSHIMOTREES)
       shimotrees = MAXSHIMOTREES;
@@ -2043,12 +2282,29 @@ void maketree()
     }
     which = 1;
     while (which <= numtrees) {
+      /*
       treeread2 (intree, &curtree.start, curtree.nodep, lengths, &trweight, 
                       &goteof, &haslengths, &spp,false,nonodes2);
+      */
+
+      /* These initializations required each time through the loop
+         since multiple trees require re-initialization */
+      nextnode         = 0;
+      goteof           = false;
+
+      treeread(intree, &curtree.start, NULL, &goteof, NULL,
+               curtree.nodep, &nextnode, NULL, &grbg, 
+               initrestmlnode, false, nonodes2);
+
+      restml_unroot(curtree.start, curtree.nodep, nonodes2);
+
       if ( outgropt )
         curtree.start = curtree.nodep[outgrno - 1]->back;
+      else
+        curtree.start = curtree.nodep[0]->back;
+
       treevaluate();
-      restml_printree();
+      restml_fprintree(outfile);
       summarize();
       if (trout) {
         col = 0;
@@ -2079,7 +2335,8 @@ void maketree()
     nextsp = 4;
     while (nextsp <= spp) {
       buildnewtip(enterorder[nextsp - 1], &curtree);
-      bestyet = - nextsp*sites*sitelength*log(4.0);
+      /* bestyet = - nextsp*sites*sitelength*log(4.0); */
+      bestyet = -DBL_MAX;
       if (smoothit)
         restml_copy_(&curtree, &priortree);
       addtraverse(curtree.nodep[enterorder[nextsp - 1] - 1]->back,
@@ -2094,7 +2351,7 @@ void maketree()
           smooth (curtree.start->back);
         }
         smoothit = false;
-        bestyet = curtree.likelihood;
+        /* bestyet = curtree.likelihood; */
       }
       if (progress)
         writename(nextsp - 1, 1, enterorder);
@@ -2134,7 +2391,7 @@ void maketree()
       if (njumble > 1)
         restml_copy_(&bestree2, &curtree);
       curtree.start = curtree.nodep[outgrno - 1]->back;
-      restml_printree();
+      restml_fprintree(outfile);
       summarize();
       if (trout) {
         col = 0;
@@ -2197,6 +2454,7 @@ int main(int argc, Char *argv[])
     for (jumb = 1; jumb <= njumble; jumb++)
       maketree();
   }
+  cleanup();
   FClose(infile);
   FClose(outfile);
   FClose(outtree);
