@@ -1,12 +1,12 @@
-#include "phylip.h"
-#include "seq.h"
-
 /* version 3.6. (c) Copyright 1993-2005 by the University of Washington.
    Written by Joseph Felsenstein, Akiko Fuseki, Sean Lamont, Andrew Keeffe,
    and Doug Buxton.
    Permission is granted to copy and use this program provided no fee is
    charged for it and provided that this copyright notice is not removed. */
 
+#include "phylip.h"
+#include "seq.h"
+
 typedef enum {
   seqs, morphology, restsites, genefreqs
 } datatype;
@@ -22,11 +22,19 @@ void   getoptions(void);
 void   seqboot_inputnumbers(void);
 void   seqboot_inputfactors(void);
 void   inputoptions(void);
+char **matrix_char_new(long rows, long cols);
+void   matrix_char_delete(char **mat, long rows);
+double **matrix_double_new(long rows, long cols);
+void   matrix_double_delete(double **mat, long rows);
 void   seqboot_inputdata(void);
 void   allocrest(void);
+void   freerest(void);
 void   allocnew(void);
+void   freenew(void);
+void   allocnewer(long newergroups, long newersites);
 void   doinput(int argc, Char *argv[]);
 void   bootweights(void);
+void   permute_vec(long *a, long n);
 void   sppermute(long);
 void   charpermute(long, long);
 void   writedata(void);
@@ -36,32 +44,78 @@ void   writeauxdata(steptr, FILE*);
 void   writefactors(void);
 void   bootwrite(void);
 void   seqboot_inputaux(steptr, FILE*);
+void   freenewer(void);
 /* function prototypes */
 #endif
 
+/*** Config vars ***/
+/* Mutually exclusive booleans for boostrap type */
+boolean bootstrap, jackknife;
+boolean permute;        /* permute char order */
+boolean ild;            /* permute species for each char */
+boolean lockhart;       /* permute chars within species */
+boolean rewrite;
 
+boolean factors = false; /* Use factors (only with morph data) */
+
+/* Bootstrap/jackknife sample frequency */
+boolean regular = true;  /* Use 50% sampling with bootstrap/jackknife */
+double fracsample = 0.5; /* ...or user-defined sample freq, [0..inf) */
+
+/* Output format: mutually exclusive, none indicates PHYLIP */
+boolean xml = false;
+boolean nexus = false;
+
+boolean weights = false;/* Read weights file */
+boolean categories = false;/* Use categories (permuted with dataset) */
+boolean enzymes;
+boolean all;             /* All alleles present in infile? */
+boolean justwts = false; /* Write boot'd/jack'd weights, no datasets */
+boolean mixture;
+boolean ancvar;
+boolean progress = true; /* Enable progress indications */
+
+boolean firstrep; /* TODO Must this be global? */
+longer seed;
+
+/* Filehandles and paths */
+/* Usual suspects declared in phylip.c/h */
 FILE *outcatfile, *outweightfile, *outmixfile, *outancfile, *outfactfile;
 Char infilename[FNMLNGTH], outfilename[FNMLNGTH], catfilename[FNMLNGTH], outcatfilename[FNMLNGTH],
   weightfilename[FNMLNGTH], outweightfilename[FNMLNGTH], mixfilename[FNMLNGTH], outmixfilename[FNMLNGTH], ancfilename[FNMLNGTH], outancfilename[FNMLNGTH],
   factfilename[FNMLNGTH], outfactfilename[FNMLNGTH];
-long sites, loci, maxalleles, groups, newsites, newersites,
-  newgroups, newergroups, nenzymes, reps, ws, blocksize, categs, maxnewsites;
-boolean bootstrap, permute, ild, lockhart, jackknife, regular, xml, nexus,
-  weights, categories, factors, enzymes, all, justwts, progress, mixture,
-  firstrep, ancvar;
-double fracsample;
+long sites, loci, maxalleles, groups, 
+  nenzymes, reps, ws, blocksize, categs, maxnewsites;
+
 datatype data;
 seqtype seq;
-steptr oldweight, where, how_many, newwhere, newhowmany,
-  newerwhere, newerhowmany, factorr, newerfactor, mixdata, ancdata;
-steptr *charorder;
-Char *factor;
-long *alleles;
-Char **nodep;
-double **nodef;
-long **sppord;
-longer seed;
+steptr oldweight, where, how_many, mixdata, ancdata;
 
+/* Original dataset */
+/* [0..spp-1][0..sites-1] */
+Char **nodep   = NULL;           /* molecular or morph data */
+double **nodef = NULL;         /* gene freqs */
+
+Char *factor = NULL;  /* factor[sites] - direct read-in of factors file */
+long *factorr = NULL; /* [0..sites-1] => nondecreasing [1..groups] */
+
+long *alleles = NULL;
+
+/* Mapping with read-in weights eliminated
+ * Allocated once in allocnew() */
+long newsites;
+long newgroups;
+long *newwhere   = NULL;    /* Map [0..newgroups-1] => [1..newsites] */
+long *newhowmany = NULL;    /* Number of chars for each [0..newgroups-1] */
+
+/* Mapping with bootstrapped weights applied */
+/* (re)allocated by allocnewer() */
+long newersites, newergroups;
+long *newerfactor  = NULL;  /* Map [0..newersites-1] => [1..newergroups] */
+long *newerwhere   = NULL;  /* Map [0..newergroups-1] => [1..newersites] */
+long *newerhowmany = NULL;  /* Number of chars for each [0..newergroups-1] */
+long **charorder   = NULL;  /* Permutation [0..spp-1][0..newergroups-1] */
+long **sppord      = NULL;  /* Permutation [0..newergroups-1][0..spp-1] */
 
 void getoptions()
 {
@@ -124,30 +178,21 @@ void getoptions()
            lockhart ? "Permute within species" :
            regular && bootstrap ? "Bootstrap" :
            (!regular) && bootstrap ? "Partial bootstrap" :
-           "Rewrite data");
+           rewrite ? "Rewrite data" : "(unknown)" );
+    
     if (bootstrap || jackknife) {
-      printf("  ");
-      putchar('%');
-      printf("    Regular or altered sampling fraction?  ");
+      printf("  %%    Regular or altered sampling fraction?  ");
       if (regular)
         printf("regular\n");
       else {
         if (fabs(fracsample*100 - (int)(fracsample*100)) > 0.01) {
-          if (fracsample < 1)
-            printf("%2.1lf", 100.0*fracsample);
-          else
-            printf("%3.1lf", 100.0*fracsample);
-        } else { if (fracsample < 1)
-            printf("%2.0lf", 100.0*fracsample);
-          else
-            printf("%3.0lf", 100.0*fracsample);
+          printf("%.1f%% sampled\n", 100.0*fracsample);
+        } else { 
+          printf("%.0f%% sampled\n", 100.0*fracsample);
         }
-        putchar('%');
-        printf(" sampled\n");
       }
     }
-    if ((data == seqs)
-        && !(jackknife || permute || bootstrap || ild || lockhart)) {
+    if ((data == seqs) && rewrite) {
       printf("  P     PHYLIP, NEXUS, or XML output format?  %s\n",
            nexus ? "NEXUS" : xml ? "XML" : "PHYLIP");
       if (xml || ((data == seqs) && nexus)) {
@@ -159,8 +204,7 @@ void getoptions()
         }
       }
     }
-    if ((data == morphology) && !(jackknife || permute || ild
-                                  || lockhart || bootstrap))
+    if ((data == morphology) && rewrite)
       printf("  P           PHYLIP or NEXUS output format?  %s\n",
                                          nexus ? "NEXUS" : "PHYLIP");
     if (bootstrap) {
@@ -169,12 +213,12 @@ void getoptions()
       else
         printf("  B      Block size for block-bootstrapping?  %ld (regular bootstrap)\n", blocksize);
     }
-    if (bootstrap || jackknife || permute || ild || lockhart)
+    if (!rewrite)
       printf("  R                     How many replicates?  %ld\n", reps);
-    if (jackknife || bootstrap || permute) {
+    if (jackknife || bootstrap || permute || ild) {
       printf("  W              Read weights of characters?  %s\n",
           (weights ? "Yes" : "No"));
-      if(data == morphology){
+      if (data == morphology) {
         printf("  X                       Read mixture file?  %s\n",
                (mixture ? "Yes" : "No"));
         printf("  N                     Read ancestors file?  %s\n",
@@ -204,6 +248,7 @@ void getoptions()
 #ifdef WIN32
     phyFillScreenColor();
 #endif
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     uppercase(&ch);
@@ -236,24 +281,34 @@ void getoptions()
         enzymes = !enzymes;
         break;
         
-      case 'J':
-        if (permute) {
-          permute = false;
-          ild = true;
-        } else if (ild) {
-            ild = false;
-            lockhart = true;
-          } else if (lockhart)
-              lockhart = false;
-            else if (jackknife) {
-                jackknife = false;
-                permute = true;
-            } else if (bootstrap) {
-                  bootstrap = false;
-                  jackknife = true;
-              } else
-                  bootstrap = true;
-        break;
+    case 'J':
+      if (bootstrap) {
+        bootstrap = false;
+        jackknife = true;
+      } else if (jackknife) {
+        jackknife = false;
+        permute = true;
+      } else if (permute) {
+        permute = false;
+        ild = true;
+      } else if (ild) {
+        ild = false;
+        lockhart = true;
+        factors = false;
+      } else if (lockhart) {
+        lockhart = false;
+        rewrite = true;
+      } else if (rewrite) {
+        rewrite = false;
+        bootstrap = true;
+      } else {
+        assert(0); /* Shouldn't happen */
+        bootstrap
+          = true;
+        jackknife = permute = ild = lockhart = rewrite
+          = false;
+      }
+      break;
         
       case '%':
         regular = !regular;
@@ -267,6 +322,7 @@ void getoptions()
             printf(" characters?\n");
             if (data == genefreqs)
               printf(" loci?\n");
+            fflush(stdout);
             scanf("%lf%*[^\n]", &fracsample);
             getchar();
             done1 = (fracsample > 0.0);
@@ -298,7 +354,7 @@ void getoptions()
         break;
         
       case 'S':
-        if(jackknife || permute || bootstrap || ild || lockhart){
+        if(!rewrite){
           justwts = !justwts;          
         } else {
           switch (seq) {
@@ -316,6 +372,7 @@ void getoptions()
 #ifdef WIN32
           phyFillScreenColor();
 #endif
+          fflush(stdout);
           scanf("%ld%*[^\n]", &blocksize);
           getchar();
           done1 = (blocksize > 0);
@@ -334,6 +391,7 @@ void getoptions()
 #ifdef WIN32
           phyFillScreenColor();
 #endif
+          fflush(stdout);
           scanf("%ld%*[^\n]", &reps);
           getchar();
           done1 = (reps > 0);
@@ -395,12 +453,35 @@ void getoptions()
     if (bootstrap && regular)
       fracsample = 1.0;
   }
-  if (bootstrap || jackknife || permute || ild || lockhart)
+  if (!rewrite)
     initseed(&inseed, &inseed0, seed);
-  xml = xml && (data == seqs);
-  categories = categories && (data == seqs);
-  mixture = mixture && (data == morphology);
-  ancvar = ancvar && (data == morphology);
+
+  /* These warnings only appear when user has changed an option that
+   * subsequently became inapplicable */
+  if (factors && lockhart) {
+    printf("Warning: Cannot use factors when permuting within species.\n");
+    factors = false;
+  }
+  if (data != seqs) {
+    if (xml) {
+      printf("warning: XML output not available for this type of data\n");
+      xml = false;
+    }
+    if (categories) {
+      printf("warning: cannot use categories file with this type of data\n");
+      categories = false;
+    }
+  }
+  if (data != morphology) {
+    if (mixture) {
+      printf("warning: cannot use mixture file with this type of data\n");
+      mixture = false;
+    }
+    if (ancvar) {
+      printf("warning: cannot use ancestors file with this type of data\n");
+      ancvar = false;
+    }
+  }
 }  /* getoptions */
 
 
@@ -515,7 +596,7 @@ void inputoptions()
   }
   maxnewsites = groups * maxfactsize;
   allocnew();
-  for (i = 0; i < (groups); i++) {
+  for (i = 0; i < groups; i++) {
     if (oldweight[where[i] - 1] > 0) {
       newgroups++;
       newsites += how_many[i];
@@ -526,6 +607,58 @@ void inputoptions()
 }  /* inputoptions */
 
 
+char **matrix_char_new(long rows, long cols)
+{
+  char **mat;
+  long i;
+
+  assert(rows > 0); assert(cols > 0);
+
+  mat = (char **)Malloc(rows*sizeof(char *));
+  for (i = 0; i < rows; i++)
+    mat[i] = (char *)Malloc(cols*sizeof(char));
+
+  return mat;
+}
+
+
+void matrix_char_delete(char **mat, long rows)
+{
+  long i;
+  
+  assert(mat != NULL);
+  for (i = 0; i < rows; i++)
+    free(mat[i]);
+  free(mat);
+}
+
+
+double **matrix_double_new(long rows, long cols)
+{
+  double **mat;
+  long i;
+
+  assert(rows > 0); assert(cols > 0);
+  
+  mat = (double **)Malloc(rows*sizeof(double *));
+  for (i = 0; i < rows; i++)
+    mat[i] = (double *)Malloc(cols*sizeof(double));
+
+  return mat;
+}
+
+
+void matrix_double_delete(double **mat, long rows)
+{
+  long i;
+  
+  assert(mat != NULL);
+  for (i = 0; i < rows; i++)
+    free(mat[i]);
+  free(mat);
+}
+
+
 void seqboot_inputdata()
 {
   /* input the names and sequences for each species */
@@ -535,13 +668,9 @@ void seqboot_inputdata()
   boolean allread, done;
 
   if (data == genefreqs) {
-    nodef = (double **)Malloc(spp*sizeof(double *));
-    for (i = 0; i < (spp); i++)
-      nodef[i] = (double *)Malloc(sites*sizeof(double));
+    nodef = matrix_double_new(spp, sites);
   } else {
-    nodep = (Char **)Malloc(spp*sizeof(Char *));
-    for (i = 0; i < (spp); i++)
-      nodep[i] = (Char *)Malloc(sites*sizeof(Char));
+    nodep = matrix_char_new(spp, sites);
   }
   j = nmlngth + (sites + (sites - 1) / 10) / 2 - 5;
   if (j < nmlngth - 1)
@@ -635,8 +764,10 @@ void seqboot_inputdata()
       while (j <= sites && !eoff(infile)) {
         if (eoln(infile)) 
           scan_eoln(infile);
-        fscanf(infile, "%lf", &x);
-        if ((unsigned)x > 1.0) {
+        if ( fscanf(infile, "%lf", &x) != 1) {
+          printf("ERROR: Invalid value for locus %ld of species %ld\n", j, i);
+          exxit(-1);
+        } else if ((unsigned)x > 1.0) {
           printf("GENE FREQ OUTSIDE [0,1] in species %ld\n", i);
           exxit(-1);
         } else {
@@ -763,21 +894,112 @@ void allocrest()
   nayme = (naym *)Malloc(spp*sizeof(naym));
 }  /* allocrest */
 
+void freerest()
+{
+  /* Free bookkeeping arrays */
+  if (alleles)
+    free(alleles);
+  free(oldweight);
+  free(weight);
+  if (categories)
+    free(category);
+  if (mixture)
+    free(mixdata);
+  if (ancvar)
+    free(ancdata);
+  free(where);
+  free(how_many);
+  free(factor);
+  free(factorr);
+  free(nayme);
+}
+
+
 void allocnew(void)
 { /* allocate memory for arrays that depend on the lenght of the 
      output sequence*/
-  long i;
+  /* Only call this function once */
+  assert(newwhere == NULL && newhowmany == NULL); 
 
   newwhere = (steptr)Malloc(loci*sizeof(long));
   newhowmany = (steptr)Malloc(loci*sizeof(long));
-  newerwhere = (steptr)Malloc(loci*sizeof(long));
-  newerhowmany = (steptr)Malloc(loci*sizeof(long));
-  newerfactor = (steptr)Malloc(maxnewsites*maxalleles*sizeof(long));
-  charorder = (steptr *)Malloc(spp*sizeof(steptr));
-  for (i = 0; i < spp; i++)
-    charorder[i] = (steptr)Malloc(maxnewsites*sizeof(long));
+} /* allocnew */
+
+
+void freenew(void)
+{ /* free arrays allocated by allocnew() */
+  /* Only call this function once */
+  assert(newwhere != NULL);
+  assert(newhowmany != NULL);
+
+  free(newwhere);
+  free(newhowmany);
 }
 
+
+void allocnewer(long newergroups, long newersites)
+{ /* allocate memory for arrays that depend on the length of the bootstrapped
+     output sequence */
+  /* Assumes that spp remains constant */
+  static long curnewergroups = 0;
+  static long curnewersites  = 0;
+
+  long i;
+
+  if (newerwhere != NULL) {
+    if (newergroups > curnewergroups) {
+      free(newerwhere);
+      free(newerhowmany);
+      for (i = 0; i < spp; i++)
+        free(charorder[i]);
+      newerwhere = NULL;
+    }
+    if (newersites > curnewersites) {
+      free(newerfactor);
+      newerfactor = NULL;
+    }
+  }
+
+  if (charorder == NULL)
+    charorder = (steptr *)Malloc(spp*sizeof(steptr));
+
+  /* Malloc() will fail if either is 0, so add a dummy element */
+  if (newergroups == 0)
+    newergroups++;
+  if (newersites == 0)
+    newersites++;
+  
+  if (newerwhere == NULL) {
+    newerwhere = (steptr)Malloc(newergroups*sizeof(long));
+    newerhowmany = (steptr)Malloc(newergroups*sizeof(long));
+    for (i = 0; i < spp; i++)
+      charorder[i] = (steptr)Malloc(newergroups*sizeof(long));
+    curnewergroups = newergroups;
+  }
+  if (newerfactor == NULL) {
+    newerfactor = (steptr)Malloc(newersites*sizeof(long));
+    curnewersites = newersites;
+  }
+}
+
+
+void freenewer()
+{
+  /* Free memory allocated by allocnewer() */
+  /* spp must be the same as when allocnewer was called */
+  long i;
+
+  if (newerwhere) {
+    free(newerwhere);
+    free(newerhowmany);
+    free(newerfactor);
+    for (i = 0; i < spp; i++)
+      free(charorder[i]);
+    free(charorder);
+  }
+}
+
+
 void doinput(int argc, Char *argv[])
 { /* reads the input data */
   getoptions();
@@ -824,6 +1046,7 @@ void bootweights()
 { /* sets up weights by resampling data */
   long i, j, k, blocks;
   double p, q, r;
+  long grp = 0, site = 0;
 
   ws = newgroups;
   for (i = 0; i < (ws); i++)
@@ -869,49 +1092,63 @@ void bootweights()
   } else             /* case of rewriting data */
     for (i = 0; i < (newgroups); i++)
       weight[i] = 1;
-  for (i = 0; i < (newgroups); i++)
-    newerwhere[i] = 0;
-  for (i = 0; i < (newgroups); i++)
-    newerhowmany[i] = 0;
+
+  /* Count number of replicated groups */
   newergroups = 0;
-  newersites = 0;
-  for (i = 0; i < (newgroups); i++) {
-    for (j = 1; j <= (weight[i]); j++) {
-      newergroups++;
-      for (k = 1; k <= (newhowmany[i]); k++) {
-        newersites++;
-        newerfactor[newersites - 1] = newergroups;
+  newersites  = 0;
+  for (i = 0; i < newgroups; i++) {
+    newergroups += weight[i];
+    newersites  += newhowmany[i] * weight[i];
+  }
+
+  if (newergroups < 1) {
+    fprintf(stdout, "ERROR: sampling frequency or number of sites is too small\n");
+    exxit(-1);
+  }
+  
+  /* reallocate "newer" arrays, sized by output groups:
+   * newerfactor, newerwhere, newerhowmany, and charorder */
+  allocnewer(newergroups, newersites);
+  
+  /* Replicate each group i weight[i] times */
+  grp = 0;
+  site = 0;
+  for (i = 0; i < newgroups; i++) {
+    for (j = 0; j < weight[i]; j++) {
+      for (k = 0; k < newhowmany[i]; k++) {
+        newerfactor[site] = grp + 1;
+        site++;
       }
-      newerwhere[newergroups - 1] = newwhere[i];
-      newerhowmany[newergroups - 1] = newhowmany[i];
+      newerwhere[grp] = newwhere[i];
+      newerhowmany[grp] = newhowmany[i];
+      grp++;
     }
   }
 }  /* bootweights */
 
 
-void sppermute(long n)
-{ /* permute the species order as given in array sppord */
+void permute_vec(long *a, long n)
+{
   long i, j, k;
 
-  for (i = 1; i <= (spp - 1); i++) {
+  for (i = 1; i < n; i++) {
     k = (long)((i+1) * randum(seed));
-    j = sppord[n - 1][i];
-    sppord[n - 1][i] = sppord[n - 1][k];
-    sppord[n - 1][k] = j;
+    j = a[i];
+    a[i] = a[k];
+    a[k] = j;
   }
+}
+
+
+void sppermute(long n)
+{ /* permute the species order as given in array sppord */
+  permute_vec(sppord[n-1], spp);
 }  /* sppermute */
 
 
 void charpermute(long m, long n)
 { /* permute the n+1 characters of species m+1 */
-  long i, j, k;
-
-  for (i = 1; i <= (n-1); i++) {
-    k = (long)((i+1) * randum(seed));
-    j = charorder[m][i];
-    charorder[m][i] = charorder[m][k];
-    charorder[m][k] = j;
-  }
+  permute_vec(charorder[m], n);
 } /* charpermute */
 
 
@@ -937,11 +1174,10 @@ void writedata()
     else if (data == genefreqs)
       fprintf(outfile, "%5ld %5ld\n", spp, newergroups);
     else {
-      if ((data == seqs)
-          && !(bootstrap || jackknife || permute || ild || lockhart) && xml)
+      if ((data == seqs) && rewrite && xml)
         fprintf(outfile, "<alignment>\n");
       else 
-        if (!(bootstrap || jackknife || permute || ild || lockhart) && nexus) {
+        if (rewrite && nexus) {
           fprintf(outfile, "#NEXUS\n");
           fprintf(outfile, "BEGIN DATA;\n");
           fprintf(outfile, "  DIMENSIONS NTAX=%ld NCHAR=%ld;\n",
@@ -974,23 +1210,22 @@ void writedata()
     }
   }
   l = 1;
-  if ((!(bootstrap || jackknife || permute || ild || lockhart || nexus))
-       && ((data == seqs) || (data == restsites))) {
+  /* When rewriting to PHYLIP, only convert interleaved <-> sequential
+   * for molecular and restriction sites. */
+  if (( (rewrite && !nexus) )
+      && ((data == seqs) || (data == restsites))) {
     interleaved = !interleaved;
-    if (!(bootstrap || jackknife || permute || ild || lockhart) && xml)
+    if (rewrite && xml)
       interleaved = false;
   }
-  if (interleaved)
-    m = 60;
-  else
-    m = newergroups;
+  m = interleaved ? 60 : newergroups;
   do {
     if (m > newergroups)
       m = newergroups;
     for (j = 0; j < spp; j++) {
       n = 0;
       if ((l == 1) || (interleaved && nexus)) {
-        if (!(bootstrap || jackknife || permute || ild || lockhart) && xml) {
+        if (rewrite && xml) {
           fprintf(outfile, "   <sequence");
           switch (seq) {
             case (dna): fprintf(outfile, " type=\"dna\""); break;
@@ -1001,8 +1236,7 @@ void writedata()
           fprintf(outfile, "      <name>");
         }
         n2 = nmlngth;
-        if (!(bootstrap || jackknife || permute || ild || lockhart)
-            && (xml || nexus)) {
+        if (rewrite && (xml || nexus)) {
           while (nayme[j][n2-1] == ' ')
             n2--;
         }
@@ -1013,10 +1247,10 @@ void writedata()
             putc('_', outfile);
           else
             putc(nayme[j][k], outfile);
-        if (!(bootstrap || jackknife || permute || ild || lockhart) && xml)
+        if (rewrite && xml)
           fprintf(outfile, "</name>\n      <data>");
       } else {
-        if (!(bootstrap || jackknife || permute || ild || lockhart) && xml) {
+        if (rewrite && xml) {
           fprintf(outfile, "      ");
         }
       }
@@ -1028,29 +1262,28 @@ void writedata()
       for (k = l - 1; k < m; k++) {
         if (permute && j + 1 == 1)
           sppermute(newerfactor[n]);    /* we can assume chars not permuted */
-        for (n2 = -1; n2 <= (newerhowmany[k] - 2); n2++) {
+        for (n2 = -1; n2 <= (newerhowmany[charorder[j][k]] - 2); n2++) {
           n++;
           if (data == genefreqs) {
             if (n > 1 && (n & 7) == 1)
               fprintf(outfile, "\n              ");
-            x = nodef[sppord[newerfactor[charorder[j][n - 1]] - 1][j] - 1]
-                    [newerwhere[charorder[j][k]] + n2];
+            x = nodef[sppord[charorder[j][k]][j] - 1]
+                     [newerwhere[charorder[j][k]] + n2];
             fprintf(outfile, "%8.5f", x);
           } else {
-            if (!(bootstrap || jackknife || permute || ild || lockhart) && xml
-                && (n > 1) && (n % 60 == 1))
+            if (rewrite && xml && (n > 1) && (n % 60 == 1))
               fprintf(outfile, "\n            ");
             else if (!nexus && !interleaved && (n > 1) && (n % 60 == 1))
                 fprintf(outfile, "\n           ");
-            charstate = nodep[sppord[newerfactor[charorder[j][n - 1]] - 1]
-                             [j] - 1][newerwhere[charorder[j][k]] + n2];
+            charstate = nodep[sppord[charorder[j][k]][j] - 1]
+                             [newerwhere[charorder[j][k]] + n2];
             putc(charstate, outfile);
             if (n % 10 == 0 && n % 60 != 0)
               putc(' ', outfile);
           }
         }
       }
-      if (!(bootstrap || jackknife || permute || ild || lockhart ) && xml) {
+      if (rewrite && xml) {
         fprintf(outfile, "</data>\n   </sequence>\n");
       }
       putc('\n', outfile);
@@ -1063,9 +1296,9 @@ void writedata()
     }
   } while (interleaved && l <= newersites);
   if ((data == seqs) &&
-      (!(bootstrap || jackknife || permute || ild || lockhart) && xml))
+      (rewrite && xml))
     fprintf(outfile, "</alignment>\n");
-  if (!(bootstrap || jackknife || permute || ild || lockhart) && nexus)
+  if (rewrite && nexus)
     fprintf(outfile, "  ;\nEND;\n");
   for (i = 0; i < (newergroups); i++)
     free(sppord[i]);
@@ -1178,7 +1411,7 @@ void writecategories()
 
 void writeauxdata(steptr auxdata, FILE *outauxfile)
 {
-  /* write out auxiliary option data (mixtures, ancestors, ect) to
+  /* write out auxiliary option data (mixtures, ancestors, etc.) to
      appropriate file.  Samples parralel to data, or just gives one
      output entry if justwts is true */
   long k, l, m, n, n2;
@@ -1244,19 +1477,19 @@ void writeauxdata(steptr auxdata, FILE *outauxfile)
 
 void writefactors(void)
 {
-  long k, l, m, n, prevfact, writesites;
+  long i, k, l, m, n, writesites;
   char symbol;
   steptr wfactor;
+  long grp;
 
   if(!justwts || firstrep){
     if(justwts){
       writesites = sites;
       wfactor = factorr;
     } else {
-      writesites = newersites;
+      writesites = newergroups;
       wfactor = newerfactor;
     }
-    prevfact = wfactor[0];
     symbol = '+';
     if (interleaved)
       m = 60;
@@ -1268,16 +1501,16 @@ void writefactors(void)
         m = writesites;
       n = 0;
       for(k=l-1 ; k < m ; k++){
-        n++;
-        if (!interleaved && n > 1 && n % 60 == 1)
-          fprintf(outfactfile, "\n ");
-        if(prevfact != wfactor[k]){
-          symbol = (symbol == '+') ? '-' : '+';
-          prevfact = wfactor[k];
+        grp = charorder[0][k];
+        for(i = 0; i < newerhowmany[grp]; i++) {
+          putc(symbol, outfactfile);
+          n++;
+          if (!interleaved && n > 1 && n % 60 == 1)
+            fprintf(outfactfile, "\n ");
+          if (n % 10 == 0 && n % 60 != 0)
+            putc(' ', outfactfile);
         }
-        putc(symbol, outfactfile);
-        if (n % 10 == 0 && n % 60 != 0)
-          putc(' ', outfactfile);
+        symbol = (symbol == '+') ? '-' : '+';
       }
       if (interleaved) {
         l += 60;
@@ -1293,31 +1526,28 @@ void bootwrite()
 { /* does bootstrapping and writes out data sets */
   long i, j, rr, repdiv10;
 
-  if (!(bootstrap || jackknife || permute || ild || lockhart))
+  if (rewrite)
     reps = 1;
   repdiv10 = reps / 10;
   if (repdiv10 < 1)
     repdiv10 = 1;
   if (progress)
     putchar('\n');
+  firstrep = true;
   for (rr = 1; rr <= (reps); rr++) {
+    bootweights();
     for (i = 0; i < spp; i++)
-      for (j = 0; j < maxnewsites; j++)
+      for (j = 0; j < newergroups; j++)
         charorder[i][j] = j;
-    if(rr==1)
-      firstrep = true;
-    else
-      firstrep = false;
     if (ild) {
-      charpermute(0, maxnewsites);
+      charpermute(0, newergroups);
       for (i = 1; i < spp; i++)
-        for (j = 0; j < maxnewsites; j++)
+        for (j = 0; j < newergroups; j++)
           charorder[i][j] = charorder[0][j];
     }
     if (lockhart)
       for (i = 0; i < spp; i++)
-        charpermute(i, maxnewsites);
-    bootweights();
+        charpermute(i, newergroups);
     if (!justwts || permute || ild || lockhart)
       writedata();
     if (justwts && !(permute || ild || lockhart))
@@ -1330,12 +1560,12 @@ void bootwrite()
       writeauxdata(mixdata, outmixfile);
     if (ancvar)
       writeauxdata(ancdata, outancfile);
-    if (progress && (bootstrap || jackknife || permute || ild || lockhart)
-          && ((reps < 10) || rr % repdiv10 == 0)) {
+    if (progress && !rewrite && ((reps < 10) || rr % repdiv10 == 0)) {
       printf("completed replicate number %4ld\n", rr);
 #ifdef WIN32
       phyFillScreenColor();
 #endif
+      firstrep = false;
     }
   }
   if (progress) {
@@ -1382,7 +1612,21 @@ int main(int argc, Char *argv[])
   ansi = ANSICRT;
   doinput(argc, argv);
   bootwrite();
+  
+  freenewer();
+  freenew();
+  freerest();
+
+  if (nodep)
+    matrix_char_delete(nodep, spp);
+  if (nodef)
+    matrix_double_delete(nodef, spp);
+
   FClose(infile);
+  if (factors) {
+    FClose(factfile);
+    FClose(outfactfile);
+  }
   if (weights)
     FClose(weightfile);
   if (categories) {
