@@ -808,6 +808,7 @@ void output_submenu()
       else
         printf ("\n Choose one: (V,S)\n");
 
+      fflush(stdout);
       scanf("%c%*[^\n]", &ch);
       getchar();
       uppercase(&ch);
@@ -856,6 +857,7 @@ void pairing_submenu()
 
     printf ("\n Choose one: (A,P,C,L)\n");
       
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     uppercase(&ch);
@@ -1014,6 +1016,7 @@ void getoptions()
     }
 
     printf("\nAre these settings correct? (type Y or the letter for one to change)\n");
+    fflush(stdout);
     scanf("%c%*[^\n]", &ch);
     getchar();
     uppercase(&ch);
@@ -1065,7 +1068,8 @@ int main(int argc, Char *argv[])
   argv[0] = "Treedist";
 #endif
   init(argc, argv);
-  openfile(&intree, INTREE, "input tree file", "r", argv[0], intreename);
+  /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+  openfile(&intree, INTREE, "input tree file", "rb", argv[0], intreename);
   openfile(&outfile, OUTFILE, "output file", "w", argv[0], outfilename);
 
   /* Initialize option-based variables, then ask for changes regarding
@@ -1084,7 +1088,8 @@ int main(int argc, Char *argv[])
   if ((tree_pairing == ALL_IN_1_AND_2) ||
       (tree_pairing == CORR_IN_1_AND_2)){
     /* If another intree file should exist, */
-    openfile(&intree2, INTREE2, "input tree file 2", "r", 
+    /* Open in binary: ftell() is broken for UNIX line-endings under WIN32 */
+    openfile(&intree2, INTREE2, "input tree file 2", "rb", 
              argv[0], intree2name);
     trees_in_2 = countsemic(&intree2);
   }
